/******************************************************************************/
/* pro2pgsql                                                                  */
/* Copyright (c) 2006, Scott Auge                                             */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

&GLOBAL-DEFINE WRT PUT UNFORMATTED
&GLOBAL-DEFINE LOG PUT STREAM LogStream UNFORMATTED
&GLOBAL-DEFINE OWNER cOwner
&GLOBAL-DEFINE DB cDBName
&GLOBAL-DEFINE ENCODING FindParam("Transformations", "Encoding")

DEFINE VARIABLE RCS AS CHARACTER INIT "$Header: /home/appl/pro2pgsql/src/RCS/cnv2pgsql.p,v 1.4 2006/07/28 06:43:19 sauge Exp sauge $" NO-UNDO.

/************************************************************************/
/* These are the API parameters to call from a GUI, WWW, CHUI, or CLI   */
/* program.                                                             */
/************************************************************************/

DEFINE INPUT PARAMETER cOutputFile AS CHARACTER NO-UNDO.
DEFINE INPUT PARAMETER cOwner      AS CHARACTER NO-UNDO.
DEFINE INPUT PARAMETER cDbName     AS CHARACTER NO-UNDO.
DEFINE INPUT PARAMETER cConfigIni  AS CHARACTER NO-UNDO.

DEFINE VARIABLE cArrayType AS CHARACTER NO-UNDO.
DEFINE VARIABLE cFmt       AS CHARACTER NO-UNDO.
DEFINE VARIABLE i          AS INTEGER NO-UNDO.

DEFINE VARIABLE cTemp1  AS CHARACTER NO-UNDO.

{tblOperationalParameters.i}

DEFINE STREAM LogStream.
DEFINE STREAM ErrStream.

/************************************************************************/
/* Convenient instead of littering code with OperationParameters finds  */
/************************************************************************/

FUNCTION FindParam RETURNS CHARACTER
    (INPUT cSection AS CHARACTER,
     INPUT cName AS CHARACTER):

  FIND OperationalParameters NO-LOCK
  WHERE OperationalParameters.PSection = cSection
    AND OperationalParameters.PName = cName
  NO-ERROR.

  IF AVAILABLE OperationalParameters THEN RETURN OperationalParameters.PValue.

END. /* FUNCTION FindParams */

{cleanname.i}

/************************************************************************/
/* Walk the ini to find out operational parameters                      */
/************************************************************************/

IF cConfigIni = ? then cConfigIni = "config.ini".
RUN inisetbytt.p (INPUT SEARCH(cConfigIni),
                  INPUT "",
          OUTPUT TABLE OperationalParameters APPEND).

/************************************************************************/
/* Initialise variable with operational parameters                      */
/************************************************************************/

ASSIGN cArrayType = FindParam("ArrayConversion", "Type").
IF cArrayType = ? THEN ASSIGN cArrayType = "Array".

/************************************************************************/
/* Start our logging.                                                   */
/************************************************************************/

OUTPUT STREAM ErrStream TO VALUE(FindParam("Logs", "ErrorLog")) APPEND.
OUTPUT STREAM LogStream TO VALUE(FindParam("Logs", "ProcessLog")) APPEND.

{&LOG} "*********** " STRING(TODAY) " " STRING (TIME, "HH:MM:SS") "**********" SKIP.

{&LOG} "Working with parameters:" SKIP.
FOR EACH OperationalParameters:

  {&LOG} OperationalParameters.PSection SPACE(1)
         OperationalParameters.PName  SPACE(1)
     OperationalParameters.PValue
     SKIP.

END. /* FOR EACH OperationalParameters */


/************************************************************************/
/* Begin processing                                                     */
/************************************************************************/

OUTPUT TO VALUE(cOutputFile).

/************************************************************************/
/* Preamble.  A lot of hard coding in here!  You may want to check if   */
/* want it this way.                                                    */
/************************************************************************/

{&LOG} "Writing Preamble" SKIP.

{&WRT} "-- Amduus Information Works, Inc. Progress 2 PostgreSQL Kit" SKIP.
{&WRT} "-- PostgreSQL Data/Schema Dump" SKIP(1).

{&WRT} "\\connect \"template1\"" SKIP.



{&WRT} "CREATE DATABASE " {&DB} " WITH TEMPLATE = template0 OWNER = " {&OWNER} " ENCODING = '" {&ENCODING} "';" SKIP(1).

{&WRT} "\\connect " {&DB} SKIP(1).

{&WRT} "SET client_encoding = '" {&ENCODING} "';                                 " SKIP.
{&WRT} "SET check_function_bodies = false;                              " SKIP.
{&WRT} "SET client_min_messages = warning;                              " SKIP.

{&WRT} "                                                                " SKIP.
{&WRT} "--                                                              " SKIP.
{&WRT} "-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: {&OWNER}" SKIP.
{&WRT} "--                                                              " SKIP.

{&WRT} "SET search_path = public, pg_catalog;                           " SKIP.
{&WRT} "                                                                " SKIP.
{&WRT} "SET default_tablespace = '';                                    " SKIP.
{&WRT} "                                                                " SKIP.
{&WRT} "SET default_with_oids = true;                                   " SKIP(2).

{&WRT} "COMMENT ON SCHEMA public IS 'Standard public schema';           " SKIP(2).

/************************************************************************/
/* Create table entries                                                 */
/************************************************************************/

FOR EACH _File NO-LOCK WHERE _file._file-num > 0 AND _file._file-num < 10000:

    {&LOG} "Working with Table " CleanName(_FILE._FILE-NAME) SKIP.
        RUN CreateTable (BUFFER _File).

END. /* FOR EACH _File */

/************************************************************************/
/* Send column descriptions as comments on colums                       */
/************************************************************************/

{&LOG} "Writing column comments" SKIP.

FOR EACH _File NO-LOCK WHERE _file._file-num > 0 AND _file._file-num < 10000:
  FOR EACH _Field NO-LOCK
    WHERE _field._file-recid = recid( _file ) by _order:

    IF _Field._Extent = 0 OR cArrayType <> "SingleField" THEN
      {&WRT} "COMMENT ON COLUMN " CleanName(_File._File-Name) "." CleanName(_Field._Field-Name) " IS '" REPLACE(STRING(_Field._Desc), "'", "\\'") "';" SKIP(1).

    ELSE
    DO:
      ASSIGN cFmt = "".
      DO i = 1 TO LENGTH(STRING(_Field._Extent)):
        ASSIGN cFmt = cFmt + "9".
      END. /* DO i */
      DO i = 1 TO _Field._Extent:
        {&WRT} "COMMENT ON COLUMN " CleanName(_File._File-Name) "." CleanName(_Field._Field-Name) "_" STRING(i, cFmt) " IS '" REPLACE(STRING(_Field._Desc), "'", "\\'") "';" SKIP(1).
      END. /* DO i */
    END. /* ELSE */

  END. /* FOR EACH _Field */
END.

/************************************************************************/
/* Lets get started on the indices.                                     */
/* Indices are objects in PostgreSQL - so they need to be uniquely      */
/* named across a database.  We use File-Name_Index-Name to name them.  */
/************************************************************************/

{&LOG} "Writing indices!" SKIP.

FOR EACH _File NO-LOCK WHERE _file._file-num > 0 AND _file._file-num < 10000:

  FOR EACH _Index OF _File NO-LOCK:

    {&LOG} "Working Index " CleanName(_Index._index-Name) " on table " CleanName(_File._File-Name) SKIP.

    {&WRT} "CREATE " (IF _Index._Unique THEN "UNIQUE" ELSE "") " INDEX "
           CleanName(_File._File-Name) + "_" + CleanName(_Index._Index-Name)
       " ON " CleanName(_File._File-Name) " USING btree (".

    RUN ipIndexFields (BUFFER _Index, OUTPUT cTemp1).

    {&WRT} cTemp1 ");" SKIp(2).

    /* from 8.1 manual: There was formerly an ALTER INDEX OWNER variant, but */
    /* this is now ignored (with a warning). An index cannot have an owner   */
    /* different from its table's owner. Changing the table's owner          */
    /* automatically changes the index as well.                              */
    /* ** Changing index owner valid up to and including 8.0                 */
    {&WRT} "ALTER INDEX public." CleanName(_File._File-Name) + "_" + CleanName(_Index._Index-Name) " OWNER TO " {&OWNER} " ;" SKIP(2).
  END. /* FOR EACH _Index */

END.

/************************************************************************/
/* Here we go with sequences.                                           */
/* WARNING: We start with 0 - there is no way to use CURRENT-VALUE in   */
/* in 9.1C or in this context.                                          */
/************************************************************************/

FOR EACH _Sequence NO-LOCK:

 {&WRT} "CREATE SEQUENCE " CleanName(_Sequence._Seq-Name)
        " INCREMENT BY " _Sequence._Seq-Incr
        " MINVALUE " _Sequence._Seq-Min
        (IF _Sequence._Seq-Max = ? THEN " NO" ELSE "")
        " MAXVALUE "
        (IF _Sequence._Seq-Max = ? THEN "" ELSE TRIM(STRING(_Sequence._Seq-Max,">>>>>>>>9")))
        " START WITH " _Sequence._Seq-Min " ;" SKIP(2).

END. /* FOR EACH _Sequence */

/************************************************************************/
/* Housekeeping.                                                        */
/************************************************************************/

{&LOG} "Done!" SKIP.

OUTPUT CLOSE.
OUTPUT STREAM ErrStream CLOSE.
OUTPUT STREAM LogStream CLOSE.

QUIT.

/*------------------------ Internal Procedures -------------------------*/

/************************************************************************/
/* Given a Progress data type, return the mapped PostgreSQL type.       */
/************************************************************************/

FUNCTION MapType RETURNS CHARACTER (INPUT cType AS CHARACTER):

  FIND OperationalParameters NO-LOCK
  WHERE OperationalParameters.PSection = "MapTypes"
    AND OperationalParameters.PName = cType
    NO-ERROR.

  IF AVAILABLE OperationalParameters THEN RETURN OperationalParameters.PValue.

  RETURN "".

END. /* FUNCTION MapType */

/************************************************************************/
/* Procedure to write the CREATE TABLE statement.                       */
/************************************************************************/

PROCEDURE CreateTable:

  DEFINE PARAMETER BUFFER b_File FOR _File.

  {&WRT} "--" SKIP.
  {&WRT} "-- Name: TABLE " CleanName(b_File._File-Name) SKIP.
  {&WRT} "--" SKIP(2).

  {&WRT} "CREATE TABLE " CleanName(b_File._File-Name) " (" SKIP.

  RUN CreateFields (BUFFER b_File).

  {&WRT} "  amduus_rsvp " MapType ("CHARACTER") SKIP.
  
  /* additional fields for DBW Data warehouse for MFG and DBW db's */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "source                 varchar   NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "sequence_no            integer   NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "datetime_loaded        timestamp NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "loaded_by              varchar   NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "datetime_updated       timestamp,"          SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "updated_by             varchar,"            SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "active                 boolean   NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "active_from            date      NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "active_to              date      NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "data_quality_indicator integer   NOT NULL," SKIP. */
  /* {&WRT} "  " entry(1,CleanName(b_File._File-Name),"_") "_" "check_code             varchar   NOT NULL"  SKIP. */


  {&WRT} ");" SKIP (1).

  {&WRT} "ALTER TABLE public." CleanName(b_File._File-Name) " OWNER TO " {&OWNER} ";" SKIP(1).

  {&WRT} "COMMENT ON TABLE " CleanName(b_File._File-Name) " IS '" REPLACE(STRING(b_File._Desc), "'", "\\'") "';" SKIP(1).

END. /* PROCEDURE CreateTable */

/************************************************************************/
/* Loop through the fields                                              */
/************************************************************************/

PROCEDURE CreateFields:

  DEFINE PARAMETER BUFFER b_File FOR _File.

  FOR EACH _Field NO-LOCK
  WHERE _field._file-recid = recid( _file ) by _order:

    RUN CreateField (BUFFER _Field).

  END. /* FOR EACH _Field */

END.  /* PROCEDURE CreateFields */

/************************************************************************/
/* Write the actual field information                                   */
/* Note this has a trailing , - we add a field in CreateFields to act   */
/* a unique identifier.                                                 */
/* In PostgreSQL 9.0+ a varchar with no limit is limitless - see manual */
/* In PostgreSQL 8.0+ a numeric with no limit is almost limitless - see */
/* manual                                                               */
/* Updated by Peter Brown for handling arrayed fields.                  */
/************************************************************************/

PROCEDURE CreateField:

  DEFINE PARAMETER BUFFER b_Field FOR _Field.
  DEFINE VARIABLE cArrayType AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i          AS INTEGER NO-UNDO.
  DEFINE VARIABLE cFmt       AS CHARACTER NO-UNDO.

  ASSIGN cArrayType = FindParam("ArrayConversion", "Type").
  IF cArrayType = ? THEN ASSIGN cArrayType = "Array".

  IF b_Field._Extent = 0 OR cArrayType <> "SingleField" THEN
  DO:
    {&WRT} "  " CleanName(b_Field._Field-Name) " "
    MapType (b_Field._Data-Type).
    if b_Field._Extent > 0 then
    {&WRT} " " "[" + STRING(_Extent) + "]".

    {&WRT} "," SKIP.
  END. /* IF */

  ELSE
  DO:
    ASSIGN cFmt = "".
    DO i = 1 TO LENGTH(STRING(b_Field._Extent)):
      ASSIGN cFmt = cFmt + "9".
    END. /* DO i */
    DO i = 1 TO b_Field._Extent:
      {&WRT} "  " CleanName(b_Field._Field-Name) + "_" + STRING(i, cFmt) " "
      MapType (b_Field._Data-Type).
      {&WRT} "," SKIP.
    END. /* DO i */
  END. /* ELSE */

END. /* PROCEDURE CreateField */

/************************************************************************/
/* Create a comma delimited list of fields in a given index.            */
/************************************************************************/

PROCEDURE ipIndexFields:
  DEFINE PARAMETER BUFFER b_Index FOR _Index.
  DEFINE OUTPUT PARAMETER cFieldList AS CHARACTER NO-UNDO.

  ASSIGN cFieldList = "".
  FOR EACH _Index-Field OF b_Index NO-LOCK,
    EACH _Field OF _Index-Field NO-LOCK:
     ASSIGN cFieldList = cFieldLIst + CleanName(_Field._Field-Name) + ",".
  END.

  /* Remove unneeded comma */

  ASSIGN cFieldList = SUBSTRING(cFieldList, 1, LENGTH(cFieldList) - 1).

END.
