/*
 * Written by Scott Auge scott_auge@yahoo.com
 * Copyright (c) 2001 Amduus Information Works, Inc.
 *
 * Reads a .conf file and sets parameter records.  See below for more detail.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      and its contributors.
 * 4. Neither the name of Amduus nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * 
 * Allowable formats:
 * 
 * # anywhere in line represents a comment - best to have these at the
 * beginning for readability.
 * [] represents a stanza
 * parm=value represents the value under that stanza
 * 
 * Example:
 * 
 * # Example Ini File
 * [Stanza1]
 * Parm1=this
 * Parm2=that
 * 
 * [Stanza2]
 * Parm1=that
 * Parm2=this
 * 
 * Returns the value of the parm under the stanza or ? if not found.
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/pro2pgsql/src/RCS/inisetbytt.p,v 1.1 2006/07/02 19:19:06 sauge Exp $" NO-UNDO.

{tblOperationalParameters.i}

DEF INPUT PARAMETER pFileName     AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pApplication  AS CHARACTER NO-UNDO.
DEF OUTPUT PARAMETER TABLE FOR OperationalParameters.

DEF VAR lStanza       AS CHARACTER NO-UNDO.
DEF VAR pParm         AS CHARACTER NO-UNDO.
DEF VAR pValue        AS CHARACTER NO-UNDO.

DEF VAR lLine         AS CHARACTER NO-UNDO.
DEF VAR lFoundStanza  AS LOGICAL NO-UNDO.
DEF VAR lIsComment    AS LOGICAL NO-UNDO.
DEF VAR pStanza       AS CHARACTER NO-UNDO.

/* Default so that we are looking through the file */

ASSIGN lFoundStanza = FALSE.

INPUT FROM VALUE(pFileName).

REPEAT:

  IMPORT UNFORMATTED lLine.

  ASSIGN lLine = TRIM (lLine).

  /* Determine if it is a comment - if so, move on */

  RUN IsComment (INPUT lLine, OUTPUT lIsComment).
  IF lIsComment THEN NEXT.

  /* Stanza contains the grammer [stanza] - see if our line matches that */

  IF SUBSTRING(lLine, 1, 1) = "[" 
  AND SUBSTRING(lLine, LENGTH(lLine), 1) = "]" THEN DO:

    ASSIGN lStanza = SUBSTRING (lLine, 2, LENGTH(lLine) - 2).
    NEXT.

  END. /* Found a stanza */

  /* If we have gotten this far, then we have found the stanza and */
  /* are looking for the parameter and it's associated value.      */

  /* Here we have found the parameter, so close the file and pass  */
  /* it's value back to the calling program.                       */

  RUN PullParmNameValuePair (INPUT lLine, OUTPUT pParm, OUTPUT pValue).

  IF pValue <> ? THEN DO:

    RUN SetParm (INPUT pApplication,
                 INPUT lStanza,
                 INPUT pParm,
                 INPUT pValue).

  END.

END. /* REPEAT */

/* IF we have not found the parameter in the file, then close up */
/* the file and return a ? to inform the caller that we could    */
/* not find it.                                                  */

INPUT CLOSE.

ASSIGN pValue = ?.

/* ------------------- Internal Procedures ----------------------- */

/* The parameter name and the parameter value are adjacent to each */
/* other seperated by an ='s sign.  Cannot do a BEGINS because an- */
/* other parameter might begin with the same name.  This pulls the */
/* parameter out if it is one.                                     */

PROCEDURE PullParmNameValuePair:

  DEF INPUT PARAMETER  lLine  AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pParm  AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pValue AS CHARACTER NO-UNDO.

  RUN PullParmName (INPUT lLine,
                    OUTPUT pParm).

  RUN PullParmValue (INPUT lLine,
                     OUTPUT pValue).

END. /* PROCEDURE PullParmNameValuePair */


PROCEDURE PullParmName:

  DEF INPUT PARAMETER  pLine  AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pValue          AS CHARACTER NO-UNDO.

  DEF VAR lWhereEqualSign     AS INTEGER NO-UNDO.

  /* We want the string at the begnning of the ='s sign, so cut that */
  /* portion out and pass it along to the caller program.            */

  ASSIGN lWhereEqualSign = INDEX(pLine, "=").

  pValue = SUBSTRING(pLine, 1, lWhereEqualSign - 1).
 
END. /* PROCEDURE PullParmName */

PROCEDURE PullParmValue:

  DEF INPUT PARAMETER  pLine  AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pValue          AS CHARACTER NO-UNDO.
  
  DEF VAR lWhereEqualSign     AS INTEGER NO-UNDO.
  DEF VAR l                   AS INTEGER NO-UNDO.

  /* We want the string at the end of the ='s sign, so cut that */
  /* portion out and pass it along to the caller program.       */

  ASSIGN lWhereEqualSign = INDEX(pLine, "=").

  ASSIGN l = LENGTH (pLine) - lWhereEqualSign.
         pValue = SUBSTRING(pLine, lWhereEqualSign + 1, l).

END. /* PROCEDURE PullParmValue */


/* Determine if the line is a comment - later on will prune out the */
/* comment text, but since a comment symbol could be a parm value,  */
/* comments are restricted to one line for the while.               */

PROCEDURE IsComment:

  DEF INPUT PARAMETER pLine    AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pValue  AS LOGICAL NO-UNDO.

  DEF VAR i AS INTEGER NO-UNDO.
  DEF VAR l AS INTEGER NO-UNDO.

  ASSIGN l = LENGTH (pLine).

  /* Blank lines are comments */
  IF l = 0 THEN DO:
    pValue = YES.
    RETURN.
  END.

  IF INDEX (pLine, "#") <> 0 THEN DO:

    ASSIGN pValue = YES.
    RETURN.

  END. /* Found a # in the line */

  ASSIGN pValue = NO.

END.  /* PROCEDURE IsComment */

/* Set the actual parmeter record */

PROCEDURE SetParm:

  DEF INPUT PARAMETER pApplication AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER pGroupName   AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER pParmName    AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER pParmValue   AS CHARACTER NO-UNDO.

  FIND OperationalParameters EXCLUSIVE-LOCK
  WHERE /* OperationalParameters.Application = pApplication
    AND */ OperationalParameters.PSection = pGroupName
    AND OperationalParameters.PName = pParmName
  NO-ERROR.

  IF NOT AVAILABLE OperationalParameters THEN  
     CREATE OperationalParameters.

  ASSIGN
  OperationalParameters.PSection    = pGroupName
  OperationalParameters.PName     = pParmName
  OperationalParameters.PValue    = pParmValue.

END. /* PROCEDURE SetParm */
